from django.core.management import BaseCommand
from django.core.management import call_command


class Command(BaseCommand):
    help = "Pull data from Stripe and update database."

    def handle(self, *args, **kwargs):
        call_command("update_stripe_products")
        call_command("update_stripe_customers")
        call_command("update_stripe_subscriptions")



from datetime import datetime
from enum import Enum
from typing import Dict, Union, List, Optional

from pydantic import BaseModel

from .currency import StripeCurrency
from .product import StripeProduct


class RecurringInterval(str, Enum):
    MONTH = 'month'
    YEAR = 'year'
    WEEK = 'week'
    DAY = 'day'


class UsageType(str, Enum):
    METERED = 'metered'
    LICENSED = 'licensed'


class PriceType(str, Enum):
    ONE_TIME = 'one_time'
    RECURRING = 'recurring'


class StripePriceRecurring(BaseModel):
    aggregate_usage: str = None
    interval: RecurringInterval
    interval_count: Optional[int]
    usage_type: Optional[UsageType]


class StripePrice(BaseModel):
    """A single StripePrice, see https://stripe.com/docs/api/prices/object"""
    id: Optional[str]
    active: Optional[bool]
    currency: Optional[StripeCurrency]
    metadata: Optional[Dict]
    nickname: Optional[str] = None
    product: Optional[Union[str, StripeProduct]]
    recurring: Optional[StripePriceRecurring] = None
    type: Optional[PriceType]
    unit_amount: Optional[int]
    created: Optional[datetime]


class StripePrices(BaseModel):
    """List of StripePrices"""
    url: str
    has_more: bool
    data: List[StripePrice]


class StripePriceEventData(BaseModel):
    """Based on https://stripe.com/docs/api/prices/object"""
    object: StripePrice
    previous_attributes: Optional[StripePrice]



from django.core.management.base import BaseCommand

from drf_stripe.stripe_api.products import stripe_api_update_products_prices


class Command(BaseCommand):
    help = "Import Service/Feature types from Stripe"

    def add_arguments(self, parser):
        pass

    def handle(self, *args, **kwargs):
        stripe_api_update_products_prices()



from rest_framework import permissions, status
from rest_framework.generics import ListAPIView
from rest_framework.response import Response
from rest_framework.views import APIView

from drf_stripe.stripe_webhooks.handler import handle_stripe_webhook_request
from .serializers import SubscriptionSerializer, PriceSerializer, SubscriptionItemSerializer
from .stripe_api.checkout import stripe_api_create_checkout_session
from .stripe_api.customer_portal import stripe_api_create_billing_portal_session
from .stripe_api.customers import get_or_create_stripe_user
from .stripe_api.subscriptions import list_user_subscriptions, list_user_subscription_items, \
    list_subscribable_product_prices_to_user, list_all_available_product_prices


class Subscription(ListAPIView):
    """Subscription of current user"""
    permission_classes = [permissions.IsAuthenticated]
    serializer_class = SubscriptionSerializer
    pagination_class = None

    def get_queryset(self):
        return list_user_subscriptions(self.request.user.id)


class SubscriptionItems(ListAPIView):
    """SubscriptionItems of current user"""
    permission_classes = [permissions.IsAuthenticated]
    serializer_class = SubscriptionItemSerializer
    pagination_class = None

    def get_queryset(self):
        return list_user_subscription_items(self.request.user.id)


class SubscribableProductPrice(ListAPIView):
    """
    Products that can be subscribed.
    Depending on whether this request is made with a bearer token,
    Anonymous user will receive a list of product and prices available to the public.
    Authenticated user will receive a list of products and prices available to the user, excluding any product prices
    the user has already been subscribed to.
    """
    permission_classes = [permissions.AllowAny]
    serializer_class = PriceSerializer
    pagination_class = None

    def get_queryset(self):
        if self.request.user.is_anonymous:
            return list_all_available_product_prices()
        else:
            return list_subscribable_product_prices_to_user(self.request.user.id)


class CreateStripeCheckoutSession(APIView):
    """
    Provides session for using Stripe hosted Checkout page.
    """
    permission_classes = [permissions.IsAuthenticated]

    def post(self, request):
        stripe_user = get_or_create_stripe_user(user_id=request.user.id)
        price_id = request.data['price_id']

        checkout_session = stripe_api_create_checkout_session(customer_id=stripe_user.customer_id, price_id=price_id)
        return Response({'session_id': checkout_session['id']}, status=status.HTTP_200_OK)


class StripeWebhook(APIView):
    """Provides endpoint for Stripe webhooks"""
    permission_classes = [permissions.AllowAny]

    def post(self, request):
        handle_stripe_webhook_request(request)
        return Response(status=status.HTTP_200_OK)


class StripeCustomerPortal(APIView):
    """Provides redirect URL for Stripe customer portal."""

    permission_classes = [permissions.IsAuthenticated]

    def post(self, request):
        session = stripe_api_create_billing_portal_session(request.user.id)
        return Response({"url": session.url}, status=status.HTTP_200_OK)



from drf_stripe.models import Subscription, SubscriptionItem, StripeUser
from drf_stripe.stripe_models.event import StripeSubscriptionEventData


def _handle_customer_subscription_event_data(data: StripeSubscriptionEventData):
    subscription_id = data.object.id
    customer = data.object.customer
    period_start = data.object.current_period_start
    period_end = data.object.current_period_end
    cancel_at_period_end = data.object.cancel_at_period_end
    cancel_at = data.object.cancel_at
    ended_at = data.object.ended_at
    status = data.object.status
    trial_end = data.object.trial_end
    trial_start = data.object.trial_start

    stripe_user = StripeUser.objects.get(customer_id=customer)

    subscription, created = Subscription.objects.update_or_create(
        subscription_id=subscription_id,
        defaults={
            "stripe_user": stripe_user,
            "period_start": period_start,
            "period_end": period_end,
            "cancel_at": cancel_at,
            "cancel_at_period_end": cancel_at_period_end,
            "ended_at": ended_at,
            "status": status,
            "trial_end": trial_end,
            "trial_start": trial_start
        })

    subscription.items.all().delete()
    _create_subscription_items(data)


def _create_subscription_items(data: StripeSubscriptionEventData):
    for item in data.object.items.data:
        SubscriptionItem.objects.update_or_create(
            sub_item_id=item.id,
            defaults={
                "subscription_id": data.object.id,
                "price_id": item.price.id,
                "quantity": item.quantity
            }
        )



from enum import Enum


class StripeCurrency(str, Enum):
    USD = 'usd'
    AED = 'aed'
    AFN = 'afn'
    ALL = 'all'
    AMD = 'amd'
    ANG = 'ang'
    AOA = 'aoa'
    ARS = 'ars'
    AUD = 'aud'
    AWG = 'awg'
    AZN = 'azn'
    BAM = 'bam'
    BBD = 'bbd'
    BDT = 'bdt'
    BGN = 'bgn'
    BIF = 'bif'
    BMD = 'bmd'
    BND = 'bnd'
    BOB = 'bob'
    BRL = 'brl'
    BSD = 'bsd'
    BWP = 'bwp'
    BYN = 'byn'
    BZD = 'bzd'
    CAD = 'cad'
    CDF = 'cdf'
    CHF = 'chf'
    CLP = 'clf'
    CNY = 'cny'
    COP = 'cop'
    CRC = 'crc'
    CVE = 'cve'
    CZK = 'czk'
    DJF = 'djf'
    DKK = 'dkk'
    DOP = 'dop'
    DZD = 'dzd'
    EGP = 'egp'
    ETB = 'etb'
    EUR = 'eur'
    FJD = 'fjd'
    FKP = 'fkp'
    GBP = 'gbp'
    GEL = 'gel'
    GIP = 'gip'
    GMD = 'gmd'
    GNF = 'gnf'
    GTQ = 'gtq'
    GYD = 'gyd'
    HKD = 'hkd'
    HNL = 'hnl'
    HRK = 'hrk'
    HTG = 'htg'
    HUF = 'huf'
    IDR = 'idr'
    ILS = 'ils'
    INR = 'inr'
    ISK = 'isk'
    JMD = 'jmd'
    JPY = 'jpy'
    KES = 'kes'
    KGS = 'kgs'
    KHR = 'khr'
    KMF = 'kmf'
    KRW = 'krw'
    KYD = 'kyd'
    KZT = 'kzt'
    LAK = 'lak'
    LBP = 'lbp'
    LKR = 'lkr'
    LRD = 'lrd'
    LSL = 'lsl'
    MAD = 'mad'
    MDL = 'mdl'
    MGA = 'mga'
    MKD = 'mkd'
    MMK = 'mmk'
    MNT = 'mnt'
    MOP = 'mop'
    MRO = 'mro'
    MUR = 'mur'
    MVR = 'mvr'
    MWK = 'mwk'
    MXN = 'mxn'
    MYR = 'myr'
    MZN = 'mzn'
    NAD = 'mad'
    NGN = 'ngn'
    NIO = 'nio'
    NOK = 'nok'
    NPR = 'npr'
    NZD = 'nzd'
    PAB = 'pab'
    PEN = 'pen'
    PGK = 'pgk'
    PHP = 'php'
    PKR = 'pkr'
    PLN = 'pln'
    PYG = 'pyg'
    QAR = 'qar'
    RON = 'ron'
    RSD = 'rsd'
    RUB = 'rub'
    RWF = 'rwf'
    SAR = 'sar'
    SBD = 'sbd'
    SCR = 'scr'
    SEK = 'sek'
    SGD = 'sgd'
    SHP = 'shp'
    SLL = 'sll'
    SOS = 'sos'
    SRD = 'srd'
    STD = 'std'
    SZL = 'szl'
    THB = 'thb'
    TJS = 'tjs'
    TOP = 'top'
    TRY = 'try'
    TTD = 'ttd'
    TWD = 'twd'
    TZS = 'tzs'
    UAH = 'uah'
    UGX = 'ugx'
    UYU = 'uyu'
    UZS = 'uzs'
    VND = 'vnd'
    VUV = 'vuv'
    WST = 'wst'
    XAF = 'xaf'
    XCD = 'xcd'
    XOF = 'xof'
    XPF = 'xpf'
    YER = 'yer'
    ZAR = 'zar'
    ZMW = 'zmw'



from typing import List, Dict, Optional

from pydantic import BaseModel

from .currency import StripeCurrency
from .price import StripePrice


class StripeInvoiceLineItem(BaseModel):
    """Based on https://stripe.com/docs/api/invoices/line_item"""
    id: str
    amount: int
    currency: StripeCurrency
    description: str = None
    metadata: Dict
    period: Dict
    price: StripePrice
    proration: bool
    quantity: int
    type: str
    discount_amounts: Optional[List[Dict]]
    discountable: Optional[bool]
    discounts: Optional[List[str]]
    invoice_item: Optional[str]
    subscription: str


class StripeInvoiceLines(BaseModel):
    """Based on https://stripe.com/docs/api/invoices/object#invoice_object-lines"""
    data: List[StripeInvoiceLineItem]
    has_more: bool
    url: str


class StripeInvoice(BaseModel):
    """Based on https://stripe.com/docs/api/invoices/object"""
    id: str
    auto_advance: Optional[bool]
    charge: str = None
    collection_method: Optional[str]
    currency: str
    customer: str
    description: str = None
    hosted_invoice_url: Optional[str]
    lines: Optional[List[StripeInvoiceLines]]


class StripeInvoiceEventData(BaseModel):
    """Based on https://stripe.com/docs/api/events/object#event_object-data"""
    object: StripeInvoice
    previous_attributes: Optional[StripeInvoice]
